"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bumpReminder = void 0;
const discord_js_1 = require("discord.js");
const bumpReminder_1 = __importDefault(require("./model/bumpReminder"));
const misc_1 = require("./misc");
const error_1 = require("./error");
// ------------------------------
// ------ F U N C T I O N -------
// ------------------------------
/**
 * A Very cool bump reminder system that reminds when a bump is necessary [Only Disboard].
 *
 * **Requires you to have this in `messageCreate` and `ready` event**
 * @param client
 * @param message
 * @param options
 * @link `Documentation:` ***https://simplyd.js.org/docs/Systems/bumpReminder***
 * @example simplydjs.bumpReminder(client, message)
 */
async function bumpReminder(client, message, options) {
    try {
        const reminder = new discord_js_1.EmbedBuilder()
            .setTitle('Bump Reminder !')
            .setDescription('Its been 2 hours since last bump. Reminding the server members to bump again.')
            .setTimestamp()
            .setColor((0, misc_1.toRgb)('#406DBC'))
            .setFooter({ text: 'Do /bump to bump the server ;)' });
        const thankyou = new discord_js_1.EmbedBuilder()
            .setTitle('Thank you')
            .setDescription('Thank you for bumping the server. This means a lot. Will notify everyone after 2 hours')
            .setTimestamp()
            .setColor((0, misc_1.toRgb)('#06bf00'))
            .setFooter({ text: 'Next bump after 120 minutes. (2 hours)' });
        if ((!options && message) || (!options && !message)) {
            return new Promise(async (resolve) => {
                setInterval(async () => {
                    const data = await bumpReminder_1.default.find({
                        counts: []
                    });
                    data.forEach(async (dt) => {
                        if (dt.nextBump && dt.nextBump < Date.now()) {
                            dt.nextBump = undefined;
                            await dt.save().catch(() => { });
                            const cho = await client.channels.fetch(dt.channel, {
                                force: true
                            });
                            await cho.send({
                                content: message.content || '\u200b',
                                embeds: [message.embed?.remind || reminder]
                            });
                            resolve(true);
                        }
                        else
                            return;
                    });
                }, (0, misc_1.ms)('5s'));
            });
        }
        if (options?.toggle == false)
            return;
        let chid = [];
        if (options && message.channel) {
            return new Promise(async (resolve) => {
                if (Array.isArray(options?.channelId)) {
                    chid = options?.channelId;
                }
                else if (!Array.isArray(options?.channelId)) {
                    chid.push(options?.channelId);
                }
                options.embed = {
                    remind: options?.embed?.remind || reminder,
                    thank: options?.embed?.thank || thankyou
                };
                if (message.author.id === '302050872383242240') {
                    for (let i = 0; i < chid.length; i++) {
                        if (message.channel.id === chid[i]) {
                            if (message.embeds[0] &&
                                message.embeds[0].description &&
                                message.embeds[0].description.includes('Bump done')) {
                                const timeout = (0, misc_1.ms)('2h');
                                const time = Date.now() + timeout;
                                let data = await bumpReminder_1.default.findOne({
                                    channel: chid[i]
                                });
                                if (!data) {
                                    data = new bumpReminder_1.default({
                                        counts: [],
                                        guild: message.guild.id,
                                        channel: chid[i],
                                        nextBump: time
                                    });
                                    await data.save().catch(() => { });
                                }
                                data.nextBump = time;
                                await data.save().catch(() => { });
                                await message.channel.send({
                                    content: options?.content || '\u200b',
                                    embeds: [options?.embed?.thank || thankyou]
                                });
                                resolve(true);
                            }
                        }
                    }
                }
            });
        }
    }
    catch (err) {
        if (options?.strict)
            throw new error_1.SimplyError({
                function: 'bumpReminder',
                title: 'An Error occured when running the function ',
                tip: err.stack
            });
        else
            console.log(`SimplyError - bumpReminder | Error: ${err.stack}`);
    }
}
exports.bumpReminder = bumpReminder;
