"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.manageTicket = void 0;
const discord_js_1 = require("discord.js");
const misc_1 = require("../misc");
const error_1 = require("../error");
// ------------------------------
// ------ F U N C T I O N -------
// ------------------------------
/**
 * A Ticket Handler for **simplydjs ticket system.**
 * @param interaction
 * @param options
 * @link `Documentation:` ***https://simplyd.js.org/docs/Handler/manageTicket***
 * @example simplydjs.manageTicket(interaction)
 */
async function manageTicket(interaction, options = {}) {
    return new Promise(async (resolve) => {
        const { client } = interaction;
        if (interaction.isButton()) {
            try {
                const member = interaction.member;
                // ------------------------------
                // ---- T I C K E T - S Y S -----
                // ------------------------------
                if (interaction.customId === 'create_ticket') {
                    await interaction.deferReply({ ephemeral: true });
                    let name = options?.ticketname || `ticket_{tag}`;
                    name = name
                        .replaceAll('{username}', member.user.username)
                        .replaceAll('{tag}', member.user.tag)
                        .replaceAll('{id}', member.user.id);
                    const topic = `Ticket has been opened by <@${member.user.id}>`;
                    const existing = interaction.guild.channels.cache.find((ch) => ch.topic === topic);
                    if (existing) {
                        await interaction.editReply({
                            content: `You have an existing ticket opened (${existing.toString()}). Close it before creating a new one.`
                        });
                    }
                    else if (!existing) {
                        let chparent;
                        const category = interaction.guild.channels.cache.get(options?.category);
                        if (!category) {
                            chparent = null;
                        }
                        chparent = category;
                        const pingRolePermissions = [];
                        const roles = [];
                        options?.pingRoles?.forEach((r) => {
                            roles.push(r);
                            pingRolePermissions.push({
                                id: r,
                                allow: [
                                    discord_js_1.PermissionFlagsBits.ViewChannel,
                                    discord_js_1.PermissionFlagsBits.SendMessages,
                                    discord_js_1.PermissionFlagsBits.ReadMessageHistory
                                ]
                            });
                        });
                        const channel = await interaction.guild.channels.create({
                            name: name,
                            type: discord_js_1.ChannelType.GuildText,
                            topic: topic,
                            parent: chparent,
                            nsfw: false,
                            permissionOverwrites: [
                                ...pingRolePermissions,
                                {
                                    id: interaction.guild.roles.everyone,
                                    deny: [
                                        discord_js_1.PermissionFlagsBits.ViewChannel,
                                        discord_js_1.PermissionFlagsBits.SendMessages,
                                        discord_js_1.PermissionFlagsBits.ReadMessageHistory
                                    ] //Deny permissions
                                },
                                {
                                    id: client.user.id,
                                    allow: [
                                        discord_js_1.PermissionFlagsBits.ViewChannel,
                                        discord_js_1.PermissionFlagsBits.SendMessages,
                                        discord_js_1.PermissionFlagsBits.ReadMessageHistory
                                    ]
                                },
                                {
                                    id: member,
                                    allow: [
                                        discord_js_1.PermissionFlagsBits.ViewChannel,
                                        discord_js_1.PermissionFlagsBits.SendMessages,
                                        discord_js_1.PermissionFlagsBits.ReadMessageHistory
                                    ]
                                }
                            ]
                        });
                        await interaction.editReply({
                            content: `🎫 Opened your support ticket in ${channel.toString()}.`
                        });
                        const embed = new discord_js_1.EmbedBuilder()
                            .setTitle(options?.embed?.title || 'Ticket Created')
                            .setDescription((options?.embed?.description ||
                            `Ticket has been raised by {user}. The support will reach you shortly.\n\n**User ID**: \`{id}\` | **User Tag**: \`{tag}\``)
                            .replaceAll('{user}', member.user.toString())
                            .replaceAll('{tag}', member.user.tag)
                            .replaceAll('{id}', member.user.id)
                            .replaceAll('{guild}', interaction.guild.name))
                            .setThumbnail(options.embed?.thumbnail || interaction.guild.iconURL())
                            .setTimestamp()
                            .setColor(options?.embed?.color || (0, misc_1.toRgb)('#406DBC'))
                            .setFooter(options?.embed?.footer
                            ? options?.embed?.footer
                            : {
                                text: '©️ Rahuletto. npm i simply-djs',
                                iconURL: 'https://i.imgur.com/XFUIwPh.png'
                            });
                        if (options?.embed?.fields)
                            embed.setFields(options?.embed?.fields);
                        if (options?.embed?.author)
                            embed.setAuthor(options?.embed?.author);
                        if (options?.embed?.image)
                            embed.setImage(options?.embed?.image);
                        if (options?.embed?.thumbnail)
                            embed.setThumbnail(options?.embed?.thumbnail);
                        if (options?.embed?.timestamp)
                            embed.setTimestamp(options?.embed?.timestamp);
                        if (options?.embed?.title)
                            embed.setTitle(options?.embed?.title);
                        if (options?.embed?.url)
                            embed.setURL(options?.embed?.url);
                        if (options?.buttons?.close?.style)
                            options.buttons.close.style = (0, misc_1.MessageButtonStyle)(options?.buttons?.close?.style);
                        if (options?.buttons?.reopen?.style)
                            options.buttons.reopen.style = (0, misc_1.MessageButtonStyle)(options?.buttons?.reopen?.style);
                        if (options?.buttons?.delete?.style)
                            options.buttons.delete.style = (0, misc_1.MessageButtonStyle)(options?.buttons?.delete?.style);
                        if (options?.buttons?.transcript?.style)
                            options.buttons.transcript.style = (0, misc_1.MessageButtonStyle)(options?.buttons?.transcript?.style);
                        const close = new discord_js_1.ButtonBuilder()
                            .setStyle(options?.buttons?.close?.style ||
                            discord_js_1.ButtonStyle.Danger)
                            .setEmoji(options?.buttons?.close?.emoji || '🔒')
                            .setLabel(options?.buttons?.close?.label || 'Close')
                            .setCustomId('close_ticket');
                        const row = new discord_js_1.ActionRowBuilder().addComponents([
                            close
                        ]);
                        channel
                            .send({
                            content: `Here is your ticket ${member.user.toString()}. | ${roles.join(',')}`,
                            embeds: [embed],
                            components: [row]
                        })
                            .then(async (msg) => {
                            await msg.pin();
                        });
                    }
                }
                else if (interaction.customId === 'close_ticket') {
                    await interaction.reply({
                        content: 'Locking the channel.',
                        ephemeral: true
                    });
                    interaction.channel.permissionOverwrites
                        .edit(member, {
                        SendMessages: false
                    })
                        .catch(() => { });
                    const deleteBtn = new discord_js_1.ButtonBuilder()
                        .setStyle(options?.buttons?.delete?.style ||
                        discord_js_1.ButtonStyle.Danger)
                        .setEmoji(options?.buttons?.delete?.emoji || '❌')
                        .setLabel(options?.buttons?.delete?.label || 'Delete')
                        .setCustomId('delete_ticket');
                    const reopenBtn = new discord_js_1.ButtonBuilder()
                        .setStyle(options?.buttons?.reopen?.style ||
                        discord_js_1.ButtonStyle.Success)
                        .setEmoji(options?.buttons?.reopen?.emoji || '🔓')
                        .setLabel(options?.buttons?.delete?.label || 'Reopen')
                        .setCustomId('open_ticket');
                    const transcriptBtn = new discord_js_1.ButtonBuilder()
                        .setStyle(options?.buttons?.transcript?.style ||
                        discord_js_1.ButtonStyle.Primary)
                        .setEmoji(options?.buttons?.transcript?.emoji || '📜')
                        .setLabel(options?.buttons?.transcript?.label || 'Transcript')
                        .setCustomId('tr_ticket');
                    const row = new discord_js_1.ActionRowBuilder().addComponents([
                        reopenBtn,
                        deleteBtn,
                        transcriptBtn
                    ]);
                    await interaction.message.edit({
                        components: [row]
                    });
                }
                else if (interaction.customId === 'tr_ticket') {
                    await interaction.deferReply({ ephemeral: true });
                    let messages = await interaction.channel.messages.fetch({
                        limit: 100
                    });
                    messages = messages.sort((a, b) => a.createdTimestamp - b.createdTimestamp);
                    const response = [];
                    messages.forEach((m) => {
                        if (m.author.bot)
                            return;
                        const attachment = m.attachments.first();
                        const url = attachment ? attachment.url : null;
                        if (url !== null) {
                            m.content = url;
                        }
                        response.push(`${m.author.username} (ID: ${m.author.id}) => ${m.content}`);
                    });
                    await interaction.editReply({
                        content: 'Collecting messages to create logs'
                    });
                    let user = interaction.channel.topic
                        .replace(`Ticket has been opened by <@`, '')
                        .replace('>', '');
                    user = await interaction.guild.members.fetch(user);
                    const attach = new discord_js_1.AttachmentBuilder(Buffer.from(response.join(`\n`), 'utf-8'), {
                        name: `${user.user.tag}.txt`
                    });
                    setTimeout(async () => {
                        await interaction.followUp({
                            content: 'Done. Generated the logs',
                            files: [attach],
                            embeds: [],
                            ephemeral: false
                        });
                    }, (0, misc_1.ms)('3s'));
                }
                else if (interaction.customId === 'delete_ticket') {
                    await interaction.deferReply({ ephemeral: false });
                    const del = new discord_js_1.ButtonBuilder()
                        .setCustomId('yes_delete')
                        .setLabel('Delete')
                        .setStyle(discord_js_1.ButtonStyle.Danger);
                    const cancel = new discord_js_1.ButtonBuilder()
                        .setCustomId('cancel')
                        .setLabel('Cancel')
                        .setStyle(discord_js_1.ButtonStyle.Success);
                    const row = new discord_js_1.ActionRowBuilder().addComponents([
                        del,
                        cancel
                    ]);
                    interaction.editReply({
                        content: 'Are you sure ?? This process is not reversible !',
                        components: [row]
                    });
                }
                else if (interaction.customId === 'yes_delete') {
                    await interaction.message.edit({
                        content: 'Deleting the channel..',
                        embeds: [],
                        components: []
                    });
                    let messages = await interaction.channel.messages.fetch({
                        limit: 100
                    });
                    const response = [];
                    messages = messages.sort((a, b) => a.createdTimestamp - b.createdTimestamp);
                    messages.forEach((m) => {
                        if (m.author.bot)
                            return;
                        const attachment = m.attachments.first();
                        const url = attachment ? attachment.url : null;
                        if (url !== null) {
                            m.content = url;
                        }
                        response.push(`${m.author.username} (ID: ${m.author.id}) => ${m.content}`);
                    });
                    let user = interaction.channel.topic
                        .replace(`Ticket has been opened by <@`, '')
                        .replace('>', '');
                    user = await interaction.guild.members.fetch(user);
                    const attach = new discord_js_1.AttachmentBuilder(Buffer.from(response.join(`\n`), 'utf-8'), {
                        name: `${user.user.tag}.txt`
                    });
                    resolve({
                        type: 'Delete',
                        channelId: interaction.channel.id,
                        user: user.user,
                        data: attach
                    });
                    if (options?.logChannelId) {
                        let ch = await client.channels.cache.get(options?.logChannelId);
                        if (ch) {
                            const log = new discord_js_1.EmbedBuilder()
                                .setTitle('Ticket deleted')
                                .setDescription(`Ticket with the name: \`${interaction.channel.name}\` got deleted. Opened by: ${user.user}.`)
                                .setTimestamp()
                                .setColor('Red');
                            await ch.send({
                                embeds: [log],
                                files: [attach]
                            });
                        }
                        else
                            return;
                    }
                    setTimeout(async () => {
                        await interaction.channel.delete();
                    }, (0, misc_1.ms)('5s'));
                }
                else if (interaction.customId === 'cancel') {
                    await interaction.deferUpdate();
                    interaction.message.edit({
                        content: 'You cancelled to delete',
                        components: []
                    });
                }
                else if (interaction.customId === 'open_ticket') {
                    await interaction.reply({
                        content: 'Unlocking the channel.',
                        ephemeral: true
                    });
                    interaction.channel.permissionOverwrites
                        .edit(member, {
                        SendMessages: true
                    })
                        .catch(() => { });
                    const close = new discord_js_1.ButtonBuilder()
                        .setStyle(options?.buttons?.close?.style ||
                        discord_js_1.ButtonStyle.Danger)
                        .setEmoji(options?.buttons?.close?.emoji || '🔒')
                        .setLabel(options?.buttons?.close?.label || 'Close')
                        .setCustomId('close_ticket');
                    const row = new discord_js_1.ActionRowBuilder().addComponents([
                        close
                    ]);
                    interaction.message.edit({ components: [row] });
                }
            }
            catch (err) {
                if (options?.strict)
                    throw new error_1.SimplyError({
                        function: 'manageTicket',
                        title: 'An Error occured when running the function ',
                        tip: err.stack
                    });
                else
                    console.log(`SimplyError - manageTicket | Error: ${err.stack}`);
            }
        }
        else
            return;
    });
}
exports.manageTicket = manageTicket;
