"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.https = void 0;
const https_1 = require("https");
/**
 * Inbuilt https function to replace your good ol' node-fetch and axios.
 * @param host
 * @param endpoint
 * @param options
 * @link `Documentation:` ***https://simplyd.js.org/docs/Others/https***
 * @example simplydjs.https("postman-echo.com", "/get") // An Echo endpoint
 */
function https(host, endpoint, options = {
    method: 'GET',
    headers: { 'Content-Type': 'application/json' }
}) {
    return new Promise((resolve, reject) => {
        // Using node:https request function
        var req = (0, https_1.request)({
            hostname: host,
            path: endpoint,
            method: options.method,
            headers: options.headers
        }, async (response) => {
            // Handle any redirects
            if (response.headers.location && response.statusCode != 200)
                return resolve(await https(host, response.headers.location.replace(host, '')));
            // Data stream
            let data = '';
            response.on('error', reject);
            response.on('data', (chunk) => (data += chunk));
            response.on('end', async () => {
                try {
                    // Resolve any objects
                    resolve(JSON.parse(data));
                }
                catch (e) {
                    // Some API sends html file as error. So this throws error if there is some
                    reject(`HttpsError | An Error Occured\n\n${e}`);
                }
            });
        }).on('error', reject);
        // Write body into the request if its other than GET method
        if (options?.body)
            req.write(JSON.stringify(options.body));
        // closes the request
        req.end();
    });
}
exports.https = https;
