import { EmbedAuthorData, EmbedFooterData, ColorResolvable, APIEmbedField } from 'discord.js';
/**
 * An Object to customize any embed that is sent from the simply-djs package
 *
 * **URL** of the Type: *https://simplyd.js.org/docs/types/CustomizableEmbed*
 * @returns {CustomizableEmbed}
 *
 * @example
 * simplydjs.someFunction(interaction, {
 * 	embed: {
 * 		title: "Example Embed",
 * 		description: "This is the embed that will be sent by simplydjs",
 * 		color: simplydjs.toRgb('#406DBC')
 * 	}
 * })
 *
 */
export interface CustomizableEmbed {
    author?: EmbedAuthorData;
    title?: string;
    url?: string;
    thumbnail?: string;
    color?: ColorResolvable;
    description?: string;
    fields?: APIEmbedField[];
    image?: string;
    footer?: EmbedFooterData;
    timestamp?: Date | number | null;
}
